package ti.modules.titanium.urbanairship;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import com.urbanairship.AirshipConfigOptions;
import com.urbanairship.UAirship;
import com.urbanairship.push.PushManager;
import com.urbanairship.push.PushPreferences;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import org.appcelerator.kroll.KrollModule;
import org.appcelerator.kroll.KrollProxy;
import org.appcelerator.kroll.common.Log;
import org.appcelerator.titanium.TiApplication;
import org.appcelerator.titanium.TiProperties;

public class UrbanAirshipModule extends KrollModule
{
  private static final String LCAT = "UrbanAirshipModule";
  private static final String PROPERTY_PREFIX = "UrbanAirship.";
  private static final String ModuleName = "UrbanAirship";
  public static final String EVENT_URBAN_AIRSHIP_SUCCESS = "UrbanAirship_Success";
  public static final String EVENT_URBAN_AIRSHIP_ERROR = "UrbanAirship_Error";
  public static final String EVENT_URBAN_AIRSHIP_CALLBACK = "UrbanAirship_Callback";
  private static final String PROPERTY_SHOW_APP_ON_CLICK = "showAppOnClick";
  private boolean checkCurrentIntent = true;
  private static boolean onAppCreateCalled = false;

  public UrbanAirshipModule()
  {
    super("UrbanAirship");
  }

  public static void onAppCreate(TiApplication app)
  {
    Log.d("UrbanAirshipModule", "Received onAppCreate notification");

    onAppCreateCalled = true;

    airshipTakeOff();
  }

  public void onStart(Activity activity)
  {
    super.onStart(activity);

    if (!onAppCreateCalled) {
      Log.e("UrbanAirshipModule", "Urban Airship module 'onAppCreate' method was not called during process initialization.");
      Log.e("UrbanAirshipModule", "Please review the module documentation and/or upgrade to the latest version of the module");
    }

    if (hasListeners("UrbanAirship_Callback"))
      synthesizeMessageIfNeeded();
  }

  public void listenerAdded(String type, int count, KrollProxy proxy)
  {
    super.listenerAdded(type, count, proxy);

    if ("UrbanAirship_Callback".equals(type))
      synthesizeMessageIfNeeded();
  }

  public boolean getIsFlying()
  {
    try
    {
      boolean flying = UAirship.shared().isFlying();

      if (!flying) {
        Log.e("UrbanAirshipModule", "Attempt to access Urban Airship while NOT flying. Check startup and configuration settings.");
      }
      return flying;
    } catch (Exception e) {
      Log.e("UrbanAirshipModule", "Error occurred during isFlying check!!!");
    }return false;
  }

  public Object[] getTags()
  {
    if (getIsFlying()) {
      return PushManager.shared().getTags().toArray();
    }
    return null;
  }

  public void setTags(Object[] rawTags)
  {
    if (getIsFlying()) {
      HashSet tags = new HashSet();
      for (Object rawTag : rawTags) {
        tags.add(rawTag.toString());
      }
      PushManager.shared().setTags(tags);
    }
  }

  public String getAlias()
  {
    if (getIsFlying()) {
      return PushManager.shared().getAlias();
    }
    return null;
  }

  public void setAlias(String alias)
  {
    if (getIsFlying())
      PushManager.shared().setAlias(alias);
  }

  public boolean getPushEnabled()
  {
    if (getIsFlying()) {
      return PushManager.shared().getPreferences().isPushEnabled();
    }
    return false;
  }

  public void setPushEnabled(boolean enabled)
  {
    if (getIsFlying())
      if (enabled)
        PushManager.enablePush();
      else
        PushManager.disablePush();
  }

  public boolean getSoundEnabled()
  {
    if (getIsFlying()) {
      return PushManager.shared().getPreferences().isSoundEnabled();
    }
    return false;
  }

  public void setSoundEnabled(boolean enabled)
  {
    if (getIsFlying())
      PushManager.shared().getPreferences().setSoundEnabled(enabled);
  }

  public boolean getVibrateEnabled()
  {
    if (getIsFlying()) {
      return PushManager.shared().getPreferences().isVibrateEnabled();
    }
    return false;
  }

  public void setVibrateEnabled(boolean enabled)
  {
    if (getIsFlying())
      PushManager.shared().getPreferences().setVibrateEnabled(enabled);
  }

  public String getPushId()
  {
    if (getIsFlying()) {
      return PushManager.shared().getPreferences().getPushId();
    }
    return null;
  }

  public void setPushId(String id)
  {
    if (getIsFlying())
      PushManager.shared().getPreferences().setPushId(id);
  }

  public String getC2dmId()
  {
    if (getIsFlying()) {
      return PushManager.shared().getPreferences().getC2DMId();
    }
    return null;
  }

  public void setC2dmId(String id)
  {
    if (getIsFlying())
      PushManager.shared().getPreferences().setC2DMId(id);
  }

  private static boolean launchAppOnClick()
  {
    TiProperties appProperties = TiApplication.getInstance().getAppProperties();
    return appProperties.getBool("UrbanAirship.showAppOnClick", false);
  }

  public void setShowOnAppClick(boolean enabled)
  {
    TiProperties appProperties = TiApplication.getInstance().getAppProperties();
    appProperties.setBool("UrbanAirship.showAppOnClick", enabled);
  }

  public boolean getShowOnAppClick()
  {
    return launchAppOnClick();
  }

  private static void airshipTakeOff()
  {
    Log.i("UrbanAirshipModule", "Airship taking off");
    try {
      AirshipConfigOptions options = AirshipConfigOptions.loadDefaultOptions(TiApplication.getInstance());

      options.iapEnabled = false;
      options.inProduction = true;

      UAirship.takeOff(TiApplication.getInstance(), options);

      PushManager.shared().setIntentReceiver(IntentReceiver.class);
    } catch (Exception e) {
      Log.e("UrbanAirshipModule", "Error occurred during takeoff!!!");
    }
  }

  private void synthesizeMessageIfNeeded()
  {
    if (this.checkCurrentIntent) {
      this.checkCurrentIntent = false;
      TiApplication appContext = TiApplication.getInstance();
      Activity activity = appContext.getCurrentActivity();
      if (activity != null) {
        Intent intent = activity.getIntent();
        if (intent != null) {
          String message = intent.getStringExtra("message");
          String payload = intent.getStringExtra("payload");
          if (message != null) {
            Log.d("UrbanAirshipModule", "User clicked notification (synthesized)");
            handleReceivedMessage(message, payload, Boolean.valueOf(true), Boolean.valueOf(false));
          }
        }
      }
    }
  }

  private static UrbanAirshipModule getModule() {
    TiApplication appContext = TiApplication.getInstance();
    UrbanAirshipModule uaModule = (UrbanAirshipModule)appContext.getModuleByName("UrbanAirship");

    if (uaModule == null) {
      Log.w("UrbanAirshipModule", "Urban Airship module not currently loaded");
    }
    return uaModule;
  }

  private static void launchActivity(String message, String payload) {
    TiApplication appContext = TiApplication.getInstance();

    PackageManager pm = appContext.getPackageManager();
    Intent launch = pm.getLaunchIntentForPackage(appContext.getPackageName());

    launch.addCategory("android.intent.category.LAUNCHER");
    launch.setFlags(268435456);

    launch.putExtra("message", message);
    launch.putExtra("payload", payload);

    appContext.startActivity(launch);
  }

  public static void handleReceivedMessage(String message, String payload, Boolean clicked, Boolean launchIfNeeded) {
    Log.d("UrbanAirshipModule", "Message: " + message + " Payload: " + payload);

    UrbanAirshipModule uaModule = getModule();

    if ((clicked.booleanValue()) && (launchIfNeeded.booleanValue()) && (launchAppOnClick())) {
      launchActivity(message, payload);
    }

    if (uaModule != null) {
      HashMap kd = new HashMap();
      kd.put("clicked", new Boolean(clicked.booleanValue()));
      kd.put("message", message);
      kd.put("payload", payload);

      uaModule.fireEvent("UrbanAirship_Callback", kd);
    }
  }

  public static void handleRegistrationComplete(String apid, Boolean valid) {
    Log.d("UrbanAirshipModule", "APID: " + apid + " IsValid: " + valid);

    UrbanAirshipModule uaModule = getModule();

    if (uaModule != null) {
      HashMap kd = new HashMap();
      kd.put("deviceToken", apid);
      kd.put("valid", valid);
      if (valid.booleanValue()) {
        uaModule.fireEvent("UrbanAirship_Success", kd);
      } else {
        kd.put("error", "Error occurred registering device");
        uaModule.fireEvent("UrbanAirship_Error", kd);
      }
    }
  }
}